from __future__ import annotations

import plothelp


def test_autoplot():
    data = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0]

    def plot_func(ax, data, i):
        ax.plot(data, i)

    plothelp.autogrid(
        data,
        plot_func=plot_func,
        title="Test",
    )



from __future__ import annotations

__all__ = ("linear_fade", "linear_fade_2D")

from functools import partial
from typing import Tuple, Union

import jax.numpy as jnp
from jax import vmap
from matplotlib.colors import to_rgb

color = Union[str, Tuple[float, float, float], Tuple[float, float, float, float]]


def linear_fade(c1: color, c2: color, num_points: int) -> jnp.ndarray:
    start = jnp.array(to_rgb(c1))
    end = jnp.array(to_rgb(c2))
    interp = vmap(partial(jnp.linspace, num=num_points))
    return interp(start, end).T


def linear_fade_2D(
    c1: color, c2: color, c3: color, c4: color, grid_size: tuple[int, int]
) -> jnp.ndarray:
    start = jnp.array(to_rgb(c1))
    end = jnp.array(to_rgb(c3))
    across = grid_size[0]
    interp = vmap(partial(jnp.linspace, num=across))
    res1 = interp(start, end)
    start = jnp.array(to_rgb(c2))
    end = jnp.array(to_rgb(c4))
    res2 = interp(start, end)
    down = grid_size[1]
    # interpolate between res1 and res2 along the down axis
    interp = vmap(partial(jnp.linspace, num=down))
    return interp(res1, res2).T



from __future__ import annotations

__all__ = ("autogrid",)

from typing import Any, Callable, Iterable, Sequence

import matplotlib.pyplot as plt


def autogrid(
    data: Sequence[Any],
    plot_func: Callable[[Any, Iterable[Any], int], None],
    plot_func_kwargs: dict[str, Any] | None = None,
    subplot_kwargs: dict[str, Any] | None = None,
    title: str | None = None,
    plots_per_row: int | None = None,
    figsize_scale: float = 2.0,
    outfile_name: str | None = None,
    tight_layout: bool = True,
) -> plt.Figure:
    """Utility function to visualize `plot_func` over an iterable set of `data`.

    Args:
        data: An iterable set of data to visualize.
        plot_func: A function that takes a single data element, the current axis, and
            the current index, then modifies the axis in place. The function should
            not return anything (it won't be used). Also optionally takes extra
            keyword arguments that are unpacked from the `plot_func_kwargs` dict.
        plot_func_kwargs: A dictionary of keyword arguments to pass to `plot_func`.
        subplot_kwargs: Keyword arguments to pass to `plt.subplots`.
        title: Title to use for the figure.
        plots_per_row: Number of plots to show per row (if None, will attempt to
            guess based on the number of data elements).
        figsize_scale: Factor to scale the figure size by.
        outfile_name: If not None, save the figure to this file.

    Returns:
        The figure object.
    """
    # automatically select the number of plots per row to minimise blank plots
    length = len(data)
    if plots_per_row is None:
        if length == 1:
            plots_per_row = 1
        else:
            if length < 3:
                i = 2
            else:
                i = 3
            rem_dict = {}
            while (length % i != 0) and i < 10:
                rem_dict[i] = length % i
                i += 1
            if i < 10:
                plots_per_row = i
            else:
                plots_per_row = max(rem_dict, key=rem_dict.get)  # type: ignore[arg-type]
                # (https://github.com/python/mypy/issues/6692) ?

    rem = length % plots_per_row
    if rem == 0:
        rows = int(length / plots_per_row)
    else:
        rows = int(length / plots_per_row) + 1
    plt.rc(
        "figure",
        figsize=[plots_per_row * figsize_scale, rows * figsize_scale],
        facecolor="w",
    )
    if subplot_kwargs is None:
        subplot_kwargs = {}
    if plot_func_kwargs is None:
        plot_func_kwargs = {}

    fig, axes = plt.subplots(nrows=rows, ncols=plots_per_row, dpi=120, **subplot_kwargs)

    row = 0
    column = 0
    axes_ravel = axes.ravel()
    for i, datum in enumerate(data):
        if i % plots_per_row == 0 and i != 0:
            row += 1
            column -= plots_per_row

        # plotting callback
        ax = axes_ravel[i]
        plot_func(ax, datum, i, **plot_func_kwargs)

        column += 1

    if rows > 1:
        while column % plots_per_row != 0:
            fig.delaxes(axes[row, column])
            column += 1
    if title is not None:
        plt.suptitle(title)
    if tight_layout:
        plt.tight_layout()
        if outfile_name is not None:
            plt.savefig(outfile_name, bbox_inches="tight")
    else:
        if outfile_name is not None:
            plt.savefig(outfile_name)
    return fig



"""
Copyright (c) 2022 Nathan . All rights reserved.

plothelp: Simple auto-grid pyplot util.
"""


from __future__ import annotations

from plothelp.colours import linear_fade, linear_fade_2D
from plothelp.plot import autogrid

from ._version import version as __version__

__all__ = ("__version__", "autogrid", "linear_fade", "linear_fade_2D")



#!/usr/bin/env python
# Copyright (c) 2022, Nathan
#
# Distributed under the 3-clause BSD license, see accompanying file LICENSE
# or https://github.com/phinate/plothelp for details.

from __future__ import annotations

from setuptools import setup

setup()

# This file is optional, on recent versions of pip you can remove it and even
# still get editable installs.



from __future__ import annotations

import shutil
from pathlib import Path

import nox

DIR = Path(__file__).parent.resolve()

nox.options.sessions = ["lint", "pylint", "tests"]


@nox.session
def lint(session: nox.Session) -> None:
    """
    Run the linter.
    """
    session.install("pre-commit")
    session.run("pre-commit", "run", "--all-files", *session.posargs)


@nox.session
def pylint(session: nox.Session) -> None:
    """
    Run PyLint.
    """
    # This needs to be installed into the package environment, and is slower
    # than a pre-commit check
    session.install(".", "pylint")
    session.run("pylint", "src", *session.posargs)


@nox.session
def tests(session: nox.Session) -> None:
    """
    Run the unit and regular tests.
    """
    session.install(".[test]")
    session.run("pytest", *session.posargs)


@nox.session
def docs(session: nox.Session) -> None:
    """
    Build the docs. Pass "serve" to serve.
    """

    session.install(".[docs]")
    session.chdir("docs")
    session.run("sphinx-build", "-M", "html", ".", "_build")

    if session.posargs:
        if "serve" in session.posargs:
            print("Launching docs at http://localhost:8000/ - use Ctrl-C to quit")
            session.run("python", "-m", "http.server", "8000", "-d", "_build/html")
        else:
            session.warn("Unsupported argument to docs")


@nox.session
def build(session: nox.Session) -> None:
    """
    Build an SDist and wheel.
    """

    build_p = DIR.joinpath("build")
    if build_p.exists():
        shutil.rmtree(build_p)

    session.install("build")
    session.run("python", "-m", "build")



from __future__ import annotations

from jax import config
from matplotlib.colors import to_rgb

from plothelp import linear_fade, linear_fade_2D

# precision
config.update("jax_enable_x64", True)


def test_linear_fade():
    c1 = "red"
    c2 = "blue"
    num_points = 10
    res = linear_fade(c1, c2, num_points)
    assert res.shape == (num_points, 3)
    red = list(to_rgb(c1))
    blue = list(to_rgb(c2))
    assert res[0].tolist() == red
    assert res[-1].tolist() == blue

    c1 = (1, 0, 0)
    c2 = (0, 0, 1)
    res = linear_fade(c1, c2, num_points)
    assert res.shape == (num_points, 3)
    assert res[0].tolist() == red
    assert res[-1].tolist() == blue

    c1 = (1, 0, 0, 1)
    c2 = (0, 0, 1, 1)
    res = linear_fade(c1, c2, num_points)
    assert res.shape == (num_points, 3)
    assert res[0].tolist() == red
    assert res[-1].tolist() == blue


def test_linear_fade_2D():
    c1 = "red"
    c2 = "blue"
    c3 = "green"
    c4 = "yellow"
    grid_size = (10, 10)
    res = linear_fade_2D(c1, c2, c3, c4, grid_size)
    red = list(to_rgb(c1))
    blue = list(to_rgb(c2))
    green = list(to_rgb(c3))
    yellow = list(to_rgb(c4))
    assert res.shape == (grid_size[0], grid_size[1], 3)
    assert res[0, 0].tolist() == red
    assert res[-1, -1].tolist() == yellow
    assert res[0, -1].tolist() == blue
    assert res[-1, 0].tolist() == green

    c1 = (1, 0, 0)
    c2 = (0, 0, 1)
    c3 = (0, 1, 0)
    c4 = (1, 1, 0)
    res = linear_fade_2D(c1, c2, c3, c4, grid_size)
    assert res.shape == (grid_size[0], grid_size[1], 3)
    assert res[0, 0].tolist() == list(c1)
    assert res[-1, -1].tolist() == list(c4)
    assert res[0, -1].tolist() == list(c2)
    assert res[-1, 0].tolist() == list(c3)

    c1 = (1, 0, 0, 1)
    c2 = (0, 0, 1, 1)
    c3 = (0, 1, 0, 1)
    c4 = (1, 1, 0, 1)
    res = linear_fade_2D(c1, c2, c3, c4, grid_size)
    assert res.shape == (grid_size[0], grid_size[1], 3)
    assert res[0, 0].tolist() == list(c1[:-1])
    assert res[-1, -1].tolist() == list(c4[:-1])
    assert res[0, -1].tolist() == list(c2[:-1])
    assert res[-1, 0].tolist() == list(c3[:-1])
